<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:db="http://docbook.org/ns/docbook"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    xmlns="http://docbook.org/ns/docbook">
    
    <xsl:variable name="generateIdPrefix">__sdwordm4342cs__</xsl:variable>
    
    <xsl:output indent="no"/>

    <xsl:template match="@* | node()">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
    </xsl:template>


    <xsl:template match="*" mode="hdmIdGenerate">
        <xsl:choose>
            <xsl:when test="@xml:id">
                <xsl:value-of select="@xml:id"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$generateIdPrefix"/>
                <xsl:value-of select="generate-id()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="db:figure[db:mediaobject/db:imageobject]/db:title">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
        </xsl:copy>
        <para>
	  <xsl:if test="substring-after(parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref, '.') = 'fig'">
            <uri>
                <xsl:attribute name="xlink:href">
                    <xsl:call-template name="hdm.get.leftmost.of.seperator">
                        <xsl:with-param name="path" 
                            select="parent::db:figure/db:mediaobject/db:imageobject/db:imagedata/@fileref"/>
                        <xsl:with-param name="seperator" select="'.'"/>
                    </xsl:call-template>                  
                    <xsl:text>.pdf</xsl:text>
                </xsl:attribute>
		<xsl:text>(Animated PDF Version)</xsl:text>
            </uri>
	  </xsl:if>
        </para>
    </xsl:template>

    <xsl:template match="db:qandaset[@role='exercise']">
        
        <xsl:for-each select="db:qandadiv/db:qandaentry">
                    
        <xsl:variable name="baseId">
            <xsl:apply-templates select="." mode="hdmIdGenerate"/>
        </xsl:variable>
        <example xml:id='{$baseId}'>
            
            <xsl:for-each select="db:question">
                <title>
                    <xsl:copy-of select='@xml:id'/>
                    <xsl:text>(Exercise) </xsl:text>
                    <xsl:apply-templates select="." mode="title"/>
                </title>
                <xsl:apply-templates select="node()[not(name(.)='label')]"/>
                <para>
                    <xref xrefstyle="template:Solution see page ">
                        <xsl:attribute name="linkend">
                            <xsl:text>solution_</xsl:text>
                            <xsl:value-of select="$baseId"/>
                        </xsl:attribute>
                    </xref>
                </para>
            </xsl:for-each>   
        </example>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match="db:question" mode="title">
        <xsl:choose>
            <xsl:when test="ancestor::db:qandaset/db:title">
                <xsl:apply-templates select="ancestor::db:qandaset/db:title/*|ancestor::db:qandaset/db:title/text()"/>
            </xsl:when>
            <xsl:when test="db:label">
                <xsl:apply-templates select="db:label/text()|db:label/node()"/>
            </xsl:when>

            <xsl:otherwise>Exercise</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="db:part">
        <xsl:copy>
            <xsl:apply-templates select="@* | node()"/>
            <appendix>
                <title>
                    <xsl:text>Solutions exercises </xsl:text>
                    <xsl:value-of select="db:title" />
                </title>
                
                <table border="1" xml:id="{@xml:id}solutionsExercise">
                    <caption>Index of excersises</caption>
                    <col width="60%" />
                    <col width="40%" />
                    <tr>
                        <th>Exercise title / page</th>
                        
                        <th>Status of completion</th>
                    </tr>
                    <xsl:for-each select=".//db:qandaset[@role='exercise']/db:qandadiv/db:qandaentry">
                    <tr>
                        <td> 
                            <xref xrefstyle="template:%n, %t">
                                <xsl:attribute name="linkend">
                                    <xsl:apply-templates select="." mode="hdmIdGenerate"/>
                                </xsl:attribute>
                            </xref>
                        </td>
                        
                        <td/>
                    </tr>
                    </xsl:for-each>
                </table>
                
                <xsl:apply-templates select=".//db:qandaset[@role='exercise']/db:qandadiv/db:qandaentry" mode="appendix"/>
            </appendix>
        </xsl:copy>        
    </xsl:template>

    <xsl:template match="db:qandaentry" mode="appendix">
        <xsl:variable name="baseId">
            <xsl:apply-templates select="." mode="hdmIdGenerate"/>
        </xsl:variable>
        <section xml:id = "solution_{$baseId}">
                <title>Solution to  <xref linkend = '{$baseId}'/></title>
                <xsl:apply-templates select="db:answer/node()"/>
            </section>
    </xsl:template>
    
    <xsl:template name="hdm.get.leftmost.of.seperator">
        <xsl:param name="path"/>
        <xsl:param name="seperator"/>
        <xsl:param name="start">1</xsl:param>
        <xsl:if test="contains($path, $seperator )">
            <xsl:if test="$start='0'">
                <xsl:value-of select="$seperator"/>
            </xsl:if>
            <xsl:value-of select="substring-before($path, $seperator)"/>
            <xsl:call-template name="hdm.get.leftmost.of.seperator">
                <xsl:with-param name="path" select="substring-after($path, $seperator)"/>
                <xsl:with-param name="seperator" select="$seperator"/>
                <xsl:with-param name="start">0</xsl:with-param>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>    
    

</xsl:stylesheet>
